'use strict';

const co = require('co');
const Thread = require('../index.js').Thread;
const assert = require('assert');
const fastDiff = require('fast-diff');
const randomstring = require('randomstring');
const EQSTR = require('./webpack/lib/test').EQSTR;
const path = require('path');

var thread = new Thread(path.join(__dirname, 'out', 'export.js'));

describe('exports', function () {
  describe('.Thread', function () {

    it('create a new thread and then close it', function(){
      var thread = new Thread(path.join(__dirname, 'out', 'export.js'));
      thread.close();
      assert.ifError(thread.isRunning());
    });

    it('should be equal with the result of fast-diff', function(){
      return co(function*() {
        var good = randomstring.generate(100);
        var bad = randomstring.generate(100);
        var r = yield thread.execute('fastDiff', new Buffer(good), new Buffer(bad));
        assert.equal(r, JSON.stringify(fastDiff(good, bad)));
      });
    });

    it('should be equal with the value of EQSTR', function(){
      return co(function *() {
        var r = yield thread.execute('printHello');
        assert.equal(r, EQSTR);
      });
    });

    it('should be equal with apple+apple+', function(){
      return co(function*() {
        var r = yield thread.execute('inc', 'apple+');
        assert.equal(r, 'apple+apple+');
      });
    });

    it('should be equal with the value of EQSTR', function(){
      return co(function* () {
        var r = yield thread.execute('two', EQSTR.substr(0, 1), EQSTR.substr(1));
        assert.equal(r, EQSTR);
      });
    });

    it('should be equal with the value of EQSTR', function(){
      return co(function* () {
        var r = yield thread.execute('three',
                                      EQSTR.substr(0, 1),
                                      EQSTR.substr(1, 1),
                                      EQSTR.substr(2));
        assert.equal(r, EQSTR);
      });
    });

    it('should be equal with the value of EQSTR', function(){
      return co(function* () {
        var r = yield thread.execute('four',
                                      EQSTR.substr(0, 1),
                                      EQSTR.substr(1, 1),
                                      EQSTR.substr(2, 1),
                                      EQSTR.substr(3));
        assert.equal(r, EQSTR);
      });
    });

    it('should be equal with the value of EQSTR', function(){
      return co(function* () {
        var r = yield thread.execute('five',
                                      EQSTR.substr(0, 1),
                                      EQSTR.substr(1, 1),
                                      EQSTR.substr(2, 1),
                                      EQSTR.substr(3, 1),
                                      EQSTR.substr(4));
        assert.equal(r, EQSTR);
      });
    });

    it('should be equal with the value of EQSTR', function(){
      return co(function* () {
        var r = yield thread.execute('six',
                                      EQSTR.substr(0, 1),
                                      EQSTR.substr(1, 1),
                                      EQSTR.substr(2, 1),
                                      EQSTR.substr(3, 1),
                                      EQSTR.substr(4, 1),
                                      EQSTR.substr(5));
        assert.equal(r, EQSTR);
      });
    });

    it('should be equal with the value of EQSTR', function(){
      return co(function* () {
        var r = yield thread.execute('seven',
                                      EQSTR.substr(0, 1),
                                      EQSTR.substr(1, 1),
                                      EQSTR.substr(2, 1),
                                      EQSTR.substr(3, 1),
                                      EQSTR.substr(4, 1),
                                      EQSTR.substr(5, 1),
                                      EQSTR.substr(6));
        assert.equal(r, EQSTR);
      });
    });

    it('should be equal with the value of EQSTR', function(){
      return co(function* () {
        var r = yield thread.execute('eight',
                                      EQSTR.substr(0, 1),
                                      EQSTR.substr(1, 1),
                                      EQSTR.substr(2, 1),
                                      EQSTR.substr(3, 1),
                                      EQSTR.substr(4, 1),
                                      EQSTR.substr(5, 1),
                                      EQSTR.substr(6, 1),
                                      EQSTR.substr(7));
        assert.equal(r, EQSTR);
      });
    });
  });
});
